@@ -14,42 +14,48 @@ using namespace ldt;
 template <bool hasWeight, DiscreteChoiceModelType modelType,
           DiscreteChoiceDistType distType>
 DiscreteChoiceSearcher<hasWeight, modelType, distType>::DiscreteChoiceSearcher(
-    SearchOptions &searchOptions, const SearchItems &searchItems,
+    const SearchData &data, const SearchCombinations &combinations,
+    SearchOptions &options, const SearchItems &items,
     const SearchMetricOptions &metrics, const SearchModelChecks &checks,
-    Ti SizeG, const std::vector<std::vector<Ti>> &groupIndexMap, Ti fixFirstG,
-    const Matrix<Tv> &source, Ti numChoices,
-    const std::vector<Matrix<Tv>> &costMatrixes, unsigned int seed,
-    Newton &newtonOptions, RocOptions &aucOptions)
-    : Searcher::Searcher(searchOptions, searchItems, metrics, checks, SizeG,
-                         groupIndexMap, fixFirstG, 0) {
+    const Ti &numPartitions, const Matrix<Tv> &source, const Ti &numChoices,
+    const std::vector<Matrix<Tv>> &costMatrixes, const unsigned int &seed,
+    const Newton &newtonOptions, RocOptions &aucOptions)
+    : SearcherReg::SearcherReg(data, combinations, options, items, metrics,
+                               checks, numPartitions, false,
+                               std::vector<Ti>({0}), 1) {
+
+  if (combinations.NumFixPartitions == 0)
+    throw LdtException(
+        ErrorType::kLogic, "dc-modelset",
+        "first partition must be fixed for intercept in binomial regression.");
 
   pCostMatrixes = &costMatrixes;
-  pSource =
-      &source; // 1st is endogenous, second is intercept, 3rd (can be) weight
-  mNumChoices = numChoices;
-  Ti numObs = source.RowsCount;
-  Ti numExo = SizeG + 1; // intercept
-  Ti cols = numExo + (hasWeight ? 2 : 1);
-  Data =
-      Dataset<Tv>(numObs, cols, true); // size + endogenous, intercept, weight
-
+  pSource = &source;
   pAucOptions = &aucOptions;
 
-  if (this->pChecks->Estimation) {
-    DModel =
-        DiscreteChoice<modelType, distType>(numObs, numExo, numChoices, false);
+  Ti num_obs = source.RowsCount;
+  Ti num_exo = numPartitions;
+  Ti num_cols = this->ColIndices.size();
+
+  mNumChoices = numChoices;
+
+  Data = Dataset<Tv>(source.RowsCount, num_cols, true);
+
+  if (this->pChecks->Estimation) { // TODO: why this structure?
+    DModel = DiscreteChoice<modelType, distType>(num_obs, num_exo, numChoices,
+                                                 false);
     DModel.Optim.IterationMax = newtonOptions.IterationMax;
     DModel.Optim.TolFunction = newtonOptions.TolFunction;
     DModel.Optim.TolGradient = newtonOptions.TolGradient;
     DModel.Optim.UseLineSearch = newtonOptions.UseLineSearch;
   }
 
-  if (metrics.SimFixSize > 0) { // we estimate a simulation model
+  if (metrics.SimFixSize > 0 && metrics.MetricsOut.size() > 0) {
     Model = DiscreteChoiceSim<hasWeight, modelType, distType>(
-        numObs, cols, this->mNumChoices, metrics.TrainRatio,
+        num_obs, num_cols, this->mNumChoices, metrics.TrainRatio,
         (Ti)metrics.TrainFixSize, (Ti)costMatrixes.size(),
-        metrics.mIndexOfBrierOut != -1, metrics.mIndexOfAucOut != -1, false,
-        nullptr,
+        metrics.MetricOutIndices.at(ScoringType::kBrier) >= 0,
+        metrics.MetricOutIndices.at(ScoringType::kAuc) >= 0, false, nullptr,
         metrics.WeightedEval); // no PCA in search
     Model.Seed = seed;
     Model.SimulationMax = metrics.SimFixSize;
@@ -59,6 +65,7 @@ DiscreteChoiceSearcher<hasWeight, modelType, distType>::DiscreteChoiceSearcher(
     Model.Optim.TolGradient = newtonOptions.TolGradient;
     Model.Optim.UseLineSearch = newtonOptions.UseLineSearch;
   }
+
   this->WorkSizeI = Model.WorkSizeI;
   this->WorkSize =
       Data.StorageSize + DModel.StorageSize + Model.StorageSize +
@@ -66,88 +73,61 @@ DiscreteChoiceSearcher<hasWeight, modelType, distType>::DiscreteChoiceSearcher(
           DModel.WorkSize,
           Model.WorkSize); // don't share storage size. we will use beta, etc.
 
-  ExoIndexes = Matrix<Ti>(this->SizeG + 1, 1);
-  this->WorkSizeI += this->SizeG + 1;
-
-  Indexes.push_back(0);
-  Indexes.push_back(1);
-  if constexpr (hasWeight) {
-    Indexes.push_back(2);
-  }
-  for (Ti i = 0; i < this->SizeG; i++)
-    Indexes.push_back(0);
-
-  auto numMeas = (Ti)this->pMetrics->MetricsOut.size() +
-                 (Ti)this->pMetrics->MetricsIn.size();
-  Weights = Matrix<Tv>(numMeas, 1);
-  this->WorkSize += numMeas; // weights matrix
-
-  if (metrics.mIndexOfCostMatrixIn != -1 || metrics.mIndexOfAucIn != -1 ||
-      metrics.mIndexOfBrierIn != -1) {
+  if (metrics.MetricInIndices.at(GoodnessOfFitType::kFrequencyCost) >= 0 ||
+      metrics.MetricInIndices.at(GoodnessOfFitType::kAuc) >= 0 ||
+      metrics.MetricInIndices.at(GoodnessOfFitType::kBrier) >= 0) {
     if (hasWeight && metrics.WeightedEval)
       CostIn = std::unique_ptr<FrequencyCostBase>(
           new FrequencyCost<true>((Ti)costMatrixes.size()));
     else
       CostIn = std::unique_ptr<FrequencyCostBase>(
           new FrequencyCost<false>((Ti)costMatrixes.size()));
-    Probs = Matrix<Tv>(numObs, numChoices);
-    this->WorkSize +=
-        std::max(numObs + numChoices - 2, CostIn.get()->StorageSize) +
-        numObs * numChoices;
-  }
-  if (metrics.mIndexOfAucIn != -1) {
 
-    if (modelType == DiscreteChoiceModelType::kBinary) {
+    Probs = Matrix<Tv>(num_obs, numChoices);
+
+    this->WorkSize +=
+        std::max(num_obs + numChoices - 2, CostIn.get()->StorageSize) +
+        num_obs * numChoices;
+  }
+
+  if (metrics.MetricInIndices.at(GoodnessOfFitType::kAuc) >= 0) {
+    if (modelType != DiscreteChoiceModelType::kBinary)
       std::logic_error("not implemented discrete choice model type");
-    }
 
     if (hasWeight && metrics.WeightedEval)
-      AucIn = std::unique_ptr<RocBase>(new ROC<true, false>(numObs));
+      AucIn = std::unique_ptr<RocBase>(new ROC<true, false>(num_obs));
     else
-      AucIn = std::unique_ptr<RocBase>(new ROC<false, false>(numObs));
+      AucIn = std::unique_ptr<RocBase>(new ROC<false, false>(num_obs));
   }
 }
 
 template <bool hasWeight, DiscreteChoiceModelType modelType,
           DiscreteChoiceDistType distType>
 std::string
-DiscreteChoiceSearcher<hasWeight, modelType, distType>::EstimateOne(Tv *work,
-                                                                    Ti *workI) {
-  auto metrics = *this->pMetrics;
-
-  ExoIndexes.SetData(workI);
-  ExoIndexes.Data[0] = 1;
-  // update Indexes
-  Ti j = 2;
-  if constexpr (hasWeight) {
-    j = 3;
-  }
-  for (Ti i = 0; i < this->SizeG; i++) {
-    Indexes.at(j + i) = j + this->CurrentIndices.Data[i];
-    ExoIndexes.Data[i + 1] = this->CurrentIndices.Data[i] + 2;
-  }
+DiscreteChoiceSearcher<hasWeight, modelType, distType>::EstimateOneReg(
+    Tv *work, Ti *workI, VMatrix<Tv> &metrics, VMatrix<Tv> &type1Mean,
+    VMatrix<Tv> &type1Var, VMatrix<Ti> &extra) {
 
   Ti s = 0;
-  Weights.SetData(NAN, &work[s]);
-  s += Weights.RowsCount;
 
-  Data.Calculate(*pSource, &Indexes, &work[s]);
+  Data.Calculate(*pSource, &this->ColIndices, &work[s]);
   s += Data.StorageSize;
 
   if (this->pOptions->RequestCancel)
     return "";
 
   Ti count = Data.Result.RowsCount;
-  Ti numExo = this->SizeG + 1;
+  Ti num_exo = this->NumPartitions;
   Tv *d = Data.Result.Data;
 
-  Y.SetData(d, count,
-            1); // You cannot set y in the constructor due to NAN existence
+  Y.SetData(d, count, 1);
+  // You cannot set y in the constructor due to NAN existence
+
   if constexpr (hasWeight) {
     W.SetData(&d[count], count, 1);
-    X.SetData(&d[2 * count], count, numExo); // +1 for intercept
+    X.SetData(&d[2 * count], count, num_exo);
   } else if constexpr (true) {
-    X.SetData(&d[count], count, numExo); // +1 for intercept
+    X.SetData(&d[count], count, num_exo);
   }
 
   // there is just one endogenous and one target
@@ -159,7 +139,7 @@ DiscreteChoiceSearcher<hasWeight, modelType, distType>::EstimateOne(Tv *work,
         throw LdtException(ErrorType::kLogic, "dc-modelset",
                            "model check: minimum no. obs");
       if (this->pChecks->MinDof > 0 &&
-          this->pChecks->MinDof > count - numExo - this->mNumChoices - 2)
+          this->pChecks->MinDof > count - num_exo - this->mNumChoices - 2)
         throw LdtException(ErrorType::kLogic, "dc-modelset",
                            "model check: minimum dof");
     }
@@ -191,13 +171,13 @@ DiscreteChoiceSearcher<hasWeight, modelType, distType>::EstimateOne(Tv *work,
     }
   }
 
-  if (metrics.SimFixSize > 0) {
+  if (this->pMetrics->SimFixSize > 0 && this->pMetrics->MetricsOut.size() > 0) {
 
     if (this->pOptions->RequestCancel)
       return "";
 
     Model.Calculate(Data.Result, pCostMatrixes, &work[s],
-                    &work[s + Model.StorageSize], &workI[this->SizeG + 1],
+                    &work[s + Model.StorageSize], &workI[this->NumPartitions],
                     this->pOptions->RequestCancel, *pAucOptions,
                     this->pMetrics->SimFixSize - this->pChecks->MinOutSim,
                     nullptr, INT32_MAX);
@@ -207,22 +187,25 @@ DiscreteChoiceSearcher<hasWeight, modelType, distType>::EstimateOne(Tv *work,
   // collect metrics:
 
   if (this->pChecks->Estimation) {
-    if (metrics.mIndexOfAic != -1)
-      Weights.Set0(
-          metrics.mIndexOfAic, 0,
-          GoodnessOfFit::ToWeight(GoodnessOfFitType::kAic, DModel.Aic));
-    if (metrics.mIndexOfSic != -1)
-      Weights.Set0(
-          metrics.mIndexOfSic, 0,
-          GoodnessOfFit::ToWeight(GoodnessOfFitType::kSic, DModel.Sic));
+    auto ind = this->pMetrics->MetricInIndices.at(GoodnessOfFitType::kAic);
+    if (ind >= 0)
+      metrics.Mat.Set0(ind, 0, DModel.Aic);
+
+    ind = this->pMetrics->MetricInIndices.at(GoodnessOfFitType::kSic);
+    if (ind >= 0)
+      metrics.Mat.Set0(ind, 0, DModel.Sic);
+
+    auto costInd =
+        this->pMetrics->MetricInIndices.at(GoodnessOfFitType::kFrequencyCost);
+    auto aucInd = this->pMetrics->MetricInIndices.at(GoodnessOfFitType::kAuc);
+    auto briInd = this->pMetrics->MetricInIndices.at(GoodnessOfFitType::kBrier);
+    if (costInd > 0 || aucInd > 0 || briInd > 0) {
 
-    if (metrics.mIndexOfCostMatrixIn != -1 || metrics.mIndexOfAucIn != -1 ||
-        metrics.mIndexOfBrierIn != -1) {
       Probs.SetData(&work[s]);
       s += Probs.length();
       DModel.GetProbabilities(X, Probs, &work[s]);
 
-      if (metrics.mIndexOfBrierIn != -1) {
+      if (briInd > 0) {
 
         Tv brier = 0;
         Tv yi, wi = 1, sum = 0;
@@ -231,104 +214,76 @@ DiscreteChoiceSearcher<hasWeight, modelType, distType>::EstimateOne(Tv *work,
           i++;
           yi = Y.Data[i];
           if constexpr (hasWeight) {
-            wi = metrics.WeightedEval ? W.Data[i] : 1;
+            wi = this->pMetrics->WeightedEval ? W.Data[i] : 1;
           }
           brier += wi * std::pow(yi - (*ap), 2);
           sum += wi;
         }
         brier /= sum;
-        Weights.Set0(metrics.mIndexOfBrierIn, 0,
-                     GoodnessOfFit::ToWeight(GoodnessOfFitType::kBrier, brier));
+
+        metrics.Mat.Set0(briInd, 0, brier);
       }
 
-      if (metrics.mIndexOfCostMatrixIn != -1) {
+      if (costInd > 0) {
         if (this->pOptions->RequestCancel)
           return "";
 
         CostIn.get()->Calculate(
             *pCostMatrixes, Y, Probs,
-            hasWeight ? (metrics.WeightedEval ? &W : nullptr) : nullptr,
+            hasWeight ? (this->pMetrics->WeightedEval ? &W : nullptr) : nullptr,
             &work[s]);
-        Weights.Set0(metrics.mIndexOfCostMatrixIn, 0,
-                     GoodnessOfFit::ToWeight(GoodnessOfFitType::kFrequencyCost,
-                                             CostIn.get()->AverageRatio));
+        metrics.Mat.Set0(costInd, 0, CostIn.get()->AverageRatio);
       }
-      if (metrics.mIndexOfAucIn != -1) {
 
+      if (aucInd > 0) {
         if (this->pOptions->RequestCancel)
           return "";
 
-        AucIn.get()->Calculate(Y, Probs,
-                               hasWeight ? (metrics.WeightedEval ? &W : nullptr)
-                                         : nullptr,
-                               *pAucOptions);
-        Weights.Set0(metrics.mIndexOfAucIn, 0,
-                     GoodnessOfFit::ToWeight(GoodnessOfFitType::kAuc,
-                                             AucIn.get()->Result));
+        AucIn.get()->Calculate(
+            Y, Probs,
+            hasWeight ? (this->pMetrics->WeightedEval ? &W : nullptr) : nullptr,
+            *pAucOptions);
+        metrics.Mat.Set0(aucInd, 0, AucIn.get()->Result);
       }
     }
   }
 
-  if (metrics.SimFixSize > 0) {
-    Ti cc = (Ti)metrics.MetricsIn.size();
-    if (metrics.mIndexOfCostMatrixOut != -1)
-      Weights.Set0(cc + metrics.mIndexOfCostMatrixOut, 0,
-                   Scoring::ToWeight(ScoringType::kFrequencyCost,
-                                     Model.CostRatios.Mean()));
-    if (metrics.mIndexOfAucOut != -1)
-      Weights.Set0(cc + metrics.mIndexOfAucOut, 0,
-                   Scoring::ToWeight(ScoringType::kAuc, Model.Auc));
+  if (this->pMetrics->SimFixSize > 0 && this->pMetrics->MetricsOut.size() > 0) {
 
-    if (metrics.mIndexOfBrierOut != -1)
-      Weights.Set0(cc + metrics.mIndexOfBrierOut, 0,
-                   Scoring::ToWeight(ScoringType::kBrier, Model.BrierScore));
+    Ti j = (Ti)this->pMetrics->MetricsIn.size();
+
+    auto ind = this->pMetrics->MetricOutIndices.at(ScoringType::kFrequencyCost);
+    if (ind >= 0)
+      metrics.Mat.Set0(j + ind, 0, Model.CostRatios.Mean());
+
+    ind = this->pMetrics->MetricOutIndices.at(ScoringType::kAuc);
+    if (ind >= 0)
+      metrics.Mat.Set0(j + ind, 0, Model.Auc);
+
+    ind = this->pMetrics->MetricOutIndices.at(ScoringType::kBrier);
+    if (ind >= 0)
+      metrics.Mat.Set0(j + ind, 0, Model.BrierScore);
   }
 
   if (this->pOptions->RequestCancel)
     return "";
 
-  // keep information:
+  // Update Type1 values:
+  extra.Mat.Data[0] = (Ti)distType;
 
-  bool allNan = true;
-  Tv weight;
-  for (Ti i = 0; i < Weights.RowsCount; i++) { // evaluation index
-    weight = Weights.Data[i];
-    if (std::isnan(weight))
-      continue;
-    allNan = false;
+  if (DModel.WorkSize > 0 && this->pItems->Length1 > 0) {
 
-    auto extra_data =
-        std::unique_ptr<Ti[]>(new Ti[1]); // to save distribution type
-    auto extra = Matrix<Ti>(extra_data.get(), 1, 1);
-    extra.Data[0] = (Ti)distType;
-
-    if (this->pItems->KeepModelEvaluations) // Add Model evaluation
-    {
-      auto ek = new EstimationKeep(weight, &this->CurrentIndices, &extra);
-      this->Push0(*ek, i, 0, &ExoIndexes);
-    }
-
-    if (this->pItems->Length1 > 0) { // Add intercept, coefficients, thresholds
-      for (Ti t = 0; t < this->SizeG + 1 + this->mNumChoices - 2; t++) {
-        auto ek =
-            new EstimationKeep(weight, &this->CurrentIndices, &extra, nullptr,
-                               DModel.Beta.Data[t], DModel.BetaVar.Get0(t, t));
-        if (t == 0) // intercept
-          this->Push1(*ek, i, 0, 0);
-        else if (t <= this->SizeG) // beta parameter
-          this->Push1(*ek, i, 0, this->CurrentIndices.Data[t - 1] + 1);
-        else // threshold
-          this->Push1(*ek, i, 0, t);
-      }
-    }
-
-    if (this->pItems->Length2 > 0) { // TODO: for saving ?!
+    // we should skip those coefficients that are not present in the current
+    // estimation.
+    Ti i = -1;
+    for (const auto &b : this->CurrentIndices.Vec) {
+      i++;
+      Ti a = b - (this->pData->HasWeight ? 2 : 1);
+      type1Mean.Mat.Data[a] = DModel.Beta.Data[i];
+      type1Var.Mat.Data[a] = DModel.BetaVar.Get0(i, i);
     }
   }
 
-  if (allNan)
-    throw LdtException(ErrorType::kLogic, "dc-modelset", "all weights are NaN");
-
   return "";
 }
 
@@ -341,40 +296,36 @@ void DiscreteChoiceModelsetBase::Start(Tv *work, Ti *worki) {
 }
 
 DiscreteChoiceModelsetBase *DiscreteChoiceModelsetBase::GetFromTypes(
-    bool isBinary, bool hasWeight, SearchOptions &searchOptions,
-    SearchItems &searchItems, SearchMetricOptions &metrics,
-    SearchModelChecks &checks, const std::vector<Ti> &sizes,
+    bool isBinary, bool hasWeight, const SearchData &data,
+    const SearchCombinations &combinations, SearchOptions &options,
+    SearchItems &items, SearchMetricOptions &metrics, SearchModelChecks &checks,
     const Matrix<Tv> &source, std::vector<Matrix<Tv>> &costMatrixes,
-    std::vector<std::vector<Ti>> &groupIndexMaps, bool addLogit, bool addProbit,
-    Newton &newtonOptions, RocOptions &aucOptions) {
+    bool addLogit, bool addProbit, Newton &newtonOptions,
+    RocOptions &aucOptions) {
   DiscreteChoiceModelsetBase *modelset;
   if (isBinary) {
     if (hasWeight) {
       modelset =
           new DiscreteChoiceModelset<true, DiscreteChoiceModelType::kBinary>(
-              searchOptions, searchItems, metrics, checks, sizes, source,
-              costMatrixes, groupIndexMaps, newtonOptions, aucOptions, addLogit,
-              addProbit);
+              data, combinations, options, items, metrics, checks, source,
+              costMatrixes, newtonOptions, aucOptions, addLogit, addProbit);
     } else {
       modelset =
           new DiscreteChoiceModelset<false, DiscreteChoiceModelType::kBinary>(
-              searchOptions, searchItems, metrics, checks, sizes, source,
-              costMatrixes, groupIndexMaps, newtonOptions, aucOptions, addLogit,
-              addProbit);
+              data, combinations, options, items, metrics, checks, source,
+              costMatrixes, newtonOptions, aucOptions, addLogit, addProbit);
     }
   } else {
     if (hasWeight) {
       modelset =
           new DiscreteChoiceModelset<true, DiscreteChoiceModelType::kOrdered>(
-              searchOptions, searchItems, metrics, checks, sizes, source,
-              costMatrixes, groupIndexMaps, newtonOptions, aucOptions, addLogit,
-              addProbit);
+              data, combinations, options, items, metrics, checks, source,
+              costMatrixes, newtonOptions, aucOptions, addLogit, addProbit);
     } else {
       modelset =
           new DiscreteChoiceModelset<false, DiscreteChoiceModelType::kOrdered>(
-              searchOptions, searchItems, metrics, checks, sizes, source,
-              costMatrixes, groupIndexMaps, newtonOptions, aucOptions, addLogit,
-              addProbit);
+              data, combinations, options, items, metrics, checks, source,
+              costMatrixes, newtonOptions, aucOptions, addLogit, addProbit);
     }
   }
   return modelset;
@@ -388,11 +339,10 @@ DiscreteChoiceModelset<hasWeight, modelType>::~DiscreteChoiceModelset() {
 
 template <bool hasWeight, DiscreteChoiceModelType modelType>
 DiscreteChoiceModelset<hasWeight, modelType>::DiscreteChoiceModelset(
-    SearchOptions &searchOptions, SearchItems &searchItems,
-    SearchMetricOptions &metrics, SearchModelChecks &checks,
-    const std::vector<Ti> &sizes, const Matrix<Tv> &source,
-    std::vector<Matrix<Tv>> &costMatrixes,
-    std::vector<std::vector<Ti>> &groupIndexMaps, Newton &newtonOptions,
+    const SearchData &data, const SearchCombinations &combinations,
+    SearchOptions &options, SearchItems &items, SearchMetricOptions &metrics,
+    SearchModelChecks &checks, const Matrix<Tv> &source,
+    std::vector<Matrix<Tv>> &costMatrixes, Newton &newtonOptions,
     RocOptions &aucOptions, bool addLogit, bool addProbit) {
 
   // find numChoices
@@ -401,39 +351,35 @@ DiscreteChoiceModelset<hasWeight, modelType>::DiscreteChoiceModelset(
   if (this->mNumChoices < 2)
     throw LdtException(ErrorType::kLogic, "dc-modelset",
                        "invalid number of choices");
-  searchItems.LengthTargets = 1;
-  searchItems.LengthDependents = 1;
-  searchItems.LengthExogenouses =
+  items.LengthTargets = 1;
+  items.LengthEndogenous = 1;
+  items.LengthExogenous =
       hasWeight ? (int)(source.ColsCount - 2) : (int)(source.ColsCount - 1);
-  if (searchItems.LengthExogenouses <
-      1) // =1, means the model has just one intercept. Let estimation process
-         // throw error (if any)
+  if (items.LengthExogenous < 1) // =1, means the model has just one intercept.
+                                 // Let estimation process throw error (if any)
     throw LdtException(ErrorType::kLogic, "dc-modelset",
                        "invalid number of exogenous variables");
 
   metrics.Update(true, false);
   checks.Update(metrics);
-  searchItems.Update(metrics, searchItems.LengthTargets,
-                     searchItems.LengthDependents,
-                     searchItems.LengthExogenouses);
+  items.Update(metrics, items.LengthTargets, items.LengthEndogenous,
+               items.LengthExogenous);
 
-  // check searchItems.Length1 with the number of exogenous variables and
+  // check items.Length1 with the number of exogenous variables and
   // thresholds?!
-  if (searchItems.Length1 != 0 &&
-      searchItems.Length1 !=
-          (searchItems.LengthExogenouses + this->mNumChoices - 2))
+  if (items.Length1 != 0 &&
+      items.Length1 != (items.LengthExogenous + this->mNumChoices - 2))
     throw LdtException(
         ErrorType::kLogic, "dc-modelset",
         "inconsistent number of exogenous variables and thresholds");
-  if (searchItems.Length1 != 0 && checks.Estimation == false)
+  if (items.Length1 != 0 && checks.Estimation == false)
     throw LdtException(ErrorType::kLogic, "dc-modelset",
                        "parameters are needed. Set 'checks.Estimation = true'");
 
-  this->pItems = &searchItems;
+  this->pItems = &items;
 
   this->pSource = &source;
   this->pCostMatrixes = &costMatrixes;
-  this->pGroupIndexMap = &groupIndexMaps;
 
   // Check intercept
   if constexpr (hasWeight) {
@@ -450,9 +396,9 @@ DiscreteChoiceModelset<hasWeight, modelType>::DiscreteChoiceModelset(
   }
 
   // check group indexes and create sizes array
-  for (auto const &b : groupIndexMaps) {
+  for (auto const &b : combinations.Partitions) {
     for (auto &a : b) {
-      if (a > searchItems.LengthExogenouses)
+      if (a > items.LengthExogenous)
         throw LdtException(
             ErrorType::kLogic, "dc-modelset",
             "invalid exogenous group element (it is larger than the number "
@@ -464,15 +410,16 @@ DiscreteChoiceModelset<hasWeight, modelType>::DiscreteChoiceModelset(
   }
 
   // check cost tables
-  if (metrics.mIndexOfCostMatrixIn == -1 &&
-      metrics.mIndexOfCostMatrixOut == -1) {
+  if (metrics.MetricInIndices.at(GoodnessOfFitType::kFrequencyCost) == -1 &&
+      metrics.MetricOutIndices.at(ScoringType::kFrequencyCost) == -1) {
     if (costMatrixes.size() > 0)
       throw LdtException(ErrorType::kLogic, "dc-modelset",
                          "There is no frequency cost metric and yet "
                          "frequency cost matrix list is not "
                          "empty!");
-  } else if (metrics.mIndexOfCostMatrixIn != -1 ||
-             metrics.mIndexOfCostMatrixOut != -1) {
+  } else if (metrics.MetricInIndices.at(GoodnessOfFitType::kFrequencyCost) >
+                 0 ||
+             metrics.MetricOutIndices.at(ScoringType::kFrequencyCost) > 0) {
     if (costMatrixes.size() == 0)
       throw LdtException(ErrorType::kLogic, "dc-modelset",
                          "Frequency cost metrics are given, "
@@ -484,7 +431,7 @@ DiscreteChoiceModelset<hasWeight, modelType>::DiscreteChoiceModelset(
   }
 
   Ti co = 0;
-  for (auto const s : sizes) {
+  for (auto const s : combinations.Sizes) {
     if (s <= 0)
       throw LdtException(
           ErrorType::kLogic, "dc-modelset",
@@ -500,22 +447,22 @@ DiscreteChoiceModelset<hasWeight, modelType>::DiscreteChoiceModelset(
       this->Searchers.push_back(
           new DiscreteChoiceSearcher<hasWeight, modelType,
                                      DiscreteChoiceDistType::kLogit>(
-              searchOptions, searchItems, metrics, checks, s, groupIndexMaps, 0,
-              source, this->mNumChoices, costMatrixes, seed, newtonOptions,
+              data, combinations, options, items, metrics, checks, s, source,
+              this->mNumChoices, costMatrixes, seed, newtonOptions,
               aucOptions));
     }
     if (addProbit) {
       this->Searchers.push_back(
           new DiscreteChoiceSearcher<hasWeight, modelType,
                                      DiscreteChoiceDistType::kProbit>(
-              searchOptions, searchItems, metrics, checks, s, groupIndexMaps, 0,
-              source, this->mNumChoices, costMatrixes, seed, newtonOptions,
+              data, combinations, options, items, metrics, checks, s, source,
+              this->mNumChoices, costMatrixes, seed, newtonOptions,
               aucOptions));
     }
   }
 
-  this->Modelset = ModelSet(this->Searchers, groupIndexMaps, searchOptions,
-                            searchItems, metrics, checks);
+  this->Modelset = ModelSet(this->Searchers, data, combinations, options, items,
+                            metrics, checks);
 }
 
 // #pragma endregion
