@@ -215,7 +215,9 @@ void Matrix<Tw>::TranslateIndex(Ti index, Ti &rowIndex, Ti &colIndex) const {
 
 template <typename Tw> Tw Matrix<Tw>::Get(Ti i, Ti j) const {
   if (i >= RowsCount || j >= ColsCount || i < 0 || j < 0)
-    throw std::out_of_range("index out-of-range exception");
+    throw std::out_of_range(format(
+        "index out-of-range in get function: (i, j)=({}, {}), Dim=({}, {})", i,
+        j, RowsCount, ColsCount));
   return Get0(i, j);
 }
 
@@ -225,9 +227,9 @@ template <typename Tw> Tw Matrix<Tw>::Get0(Ti i, Ti j) const {
 
 template <typename Tw> Tw Matrix<Tw>::Get(Ti i) const {
   if (i < 0 || i >= length())
-    throw std::out_of_range("index out-of-range exception");
+    throw std::out_of_range(format(
+        "index out-of-range in get function: i={}, Length={}", i, length()));
   return Data[i];
-  ;
 }
 
 template <typename Tw> Tw Matrix<Tw>::GetVector(Ti i) const {
@@ -240,7 +242,9 @@ template <typename Tw> Tw Matrix<Tw>::GetVector(Ti i) const {
 
 template <typename Tw> void Matrix<Tw>::Set(Ti i, Ti j, Tw value) {
   if (i >= RowsCount || j >= ColsCount || i < 0 || j < 0)
-    throw std::out_of_range("index out-of-range exception");
+    throw std::out_of_range(format(
+        "index out-of-range in set function: (i, j)=({}, {}), Dim=({}, {})", i,
+        j, RowsCount, ColsCount));
   Set0(i, j, value);
 }
 
@@ -250,7 +254,8 @@ template <typename Tw> void Matrix<Tw>::Set0(Ti i, Ti j, Tw value) {
 
 template <typename Tw> void Matrix<Tw>::Set_Plus(Ti i, Ti j, Tw value) {
   if (i >= RowsCount || j >= ColsCount || i < 0 || j < 0)
-    throw std::out_of_range("index out-of-range exception");
+    throw std::out_of_range(format(
+        "index out-of-range in set function: i={}, Length={}", i, length()));
   Set_Plus0(i, j, value);
 }
 
@@ -1193,7 +1198,7 @@ void Matrix<Tw>::GetSub(Ti rowstart, Ti colstart, Ti rowcount, Ti colcount,
   auto storreqcol = colcount + storaecolstart;
 
   if (storage.RowsCount > storreqrow || storage.ColsCount > storreqcol)
-    throw std::invalid_argument("inconsistent size: 'storage'");
+    throw std::invalid_argument("inconsistent size in get sub (1). ");
 
   GetSub0(rowstart, colstart, rowcount, colcount, storage, storagerowstart,
           storaecolstart);
@@ -1215,30 +1220,58 @@ void Matrix<Tw>::GetSub(Ti firststart, Ti firstcount,
   if (exclude) {
     if (firstIsRow) {
       if (storage.RowsCount != firstcount + storagerowstart)
-        throw std::invalid_argument("inconsistent size: 'storage'");
+        throw std::out_of_range(
+            format("index out-of-range in get sub function: storage rows={}, "
+                   "row count={}, storage row start={}",
+                   storage.RowsCount, firstcount, storagerowstart));
       if (storage.ColsCount !=
           ColsCount - (Ti)secondindexes.size() + storaecolstart)
-        throw std::invalid_argument("inconsistent size: 'storage'");
+        throw std::out_of_range(format(
+            "index out-of-range in get sub function: storage columns={}, "
+            "columns count={}"
+            "column indices size={}, storage column start={}",
+            storage.ColsCount, ColsCount, secondindexes.size(),
+            storaecolstart));
     }
     if (firstIsRow == false) {
       if (storage.ColsCount != firstcount + storaecolstart)
-        throw std::invalid_argument("inconsistent size: 'storage'");
+        throw std::out_of_range(format(
+            "index out-of-range in get sub function: storage columns={}, "
+            "columns count={}, storage column start={}",
+            storage.ColsCount, firstcount, storaecolstart));
       if (storage.RowsCount !=
           RowsCount - (Ti)secondindexes.size() + storagerowstart)
-        throw std::invalid_argument("inconsistent size: 'storage'");
+        throw std::out_of_range(
+            format("index out-of-range in get sub function: storage rows={}, "
+                   "rows count={}"
+                   "row indices size={}, storage row start={}",
+                   storage.RowsCount, RowsCount, secondindexes.size(),
+                   storagerowstart));
     }
   } else {
     if (firstIsRow) {
       if (storage.RowsCount != firstcount + storagerowstart)
-        throw std::invalid_argument("inconsistent size: 'storage'");
+        throw std::out_of_range(
+            format("index out-of-range in get sub function: storage rows={}, "
+                   "rows count={}, storage row start={}",
+                   storage.RowsCount, firstcount, storagerowstart));
       if (storage.ColsCount != (Ti)secondindexes.size() + storaecolstart)
-        throw std::invalid_argument("inconsistent size: 'storage'");
+        throw std::out_of_range(format(
+            "index out-of-range in get sub function: storage columns={}, "
+            "column indices size={}, storage column start={}",
+            storage.ColsCount, secondindexes.size(), storaecolstart));
     }
     if (firstIsRow == false) {
       if (storage.ColsCount != firstcount + storaecolstart)
-        throw std::invalid_argument("inconsistent size: 'storage'");
+        throw std::out_of_range(format(
+            "index out-of-range in get sub function: storage columns={}, "
+            "columns count={}, storage row start={}",
+            storage.ColsCount, firstcount, storaecolstart));
       if (storage.RowsCount != (Ti)secondindexes.size() + storagerowstart)
-        throw std::invalid_argument("inconsistent size: 'storage'");
+        throw std::out_of_range(
+            format("index out-of-range in get sub function: storage rows={}, "
+                   "row indices size={}, storage row start={}",
+                   storage.RowsCount, secondindexes.size(), storagerowstart));
     }
   }
   GetSub0(firststart, firstcount, secondindexes, firstIsRow, storage,
