@@ -38,8 +38,9 @@ List ClusterH(NumericVector distances, std::string linkage) {
   double n = (1 + sqrt(1 + 8 * distances.length())) / 2;
   int numVariables = std::floor(n);
   if (std::abs(n - numVariables) > 1e-16 || numVariables <= 1)
-    throw std::logic_error("Invalid distance vector. It should be the lower "
-                           "triangle of a symmetric matrix.");
+    throw LdtException(ErrorType::kLogic, "R-clustering",
+                       "distance vector should be the lower "
+                       "triangle of a symmetric matrix");
 
   boost::algorithm::to_lower(linkage);
   HClusterLinkage linkage0 = FromString_HClusterLinkage(linkage.c_str());
@@ -70,7 +71,8 @@ List ClusterHGroup(NumericMatrix data, int nGroups = 2, double threshold = 0,
                    std::string correlation = "pearson") {
 
   if (threshold < 0)
-    throw std::logic_error("Invalid threshold. It cannot be negative.");
+    throw LdtException(ErrorType::kLogic, "R-clustering",
+                       "threshold cannot be negative");
 
   boost::algorithm::to_lower(distance);
   boost::algorithm::to_lower(linkage);
@@ -93,7 +95,7 @@ List ClusterHGroup(NumericMatrix data, int nGroups = 2, double threshold = 0,
   if (group.get()->NaNDistanceFound)
     Rf_warning(
         "NA distance found and converted to zero. If you are using a "
-        "correlation based distance, make sure variables are not constant.");
+        "correlation based distance, make sure variables are not constant");
 
   auto groups = std::vector<IntegerVector>();
 
